@@ -235,9 +235,11 @@ int copy_file(const char *dest, const char *src)
 		goto done;
 	}
 
-	ssize_t cnt;
+	assert(ret == 0);
+	size_t cnt;
 	while ((cnt = fread(buf, sizeof(*buf), BUFSIZE, from)) != 0 &&
-	       (ret = (fwrite(buf, sizeof(*buf), cnt, file) == cnt))) {
+	       (ret = (fwrite(buf, sizeof(*buf), cnt, file) == cnt)) &&
+	       (feof(from) | ferror(from)) == 0) {
 	}
 
 	ret = !ret || ferror(from);
