@@ -82,7 +82,7 @@ static inline const char *log_conn_info(knotd_query_proto_t proto, bool pool)
 	case KNOTD_QUERY_PROTO_QUIC:
 		return pool ? " QUIC/0-RTT" : " QUIC";
 	case KNOTD_QUERY_PROTO_TLS:
-		return " TLS";
+		return pool ? " TLS/0-RTT" : " TLS";
 	default:
 		return "";
 	}
