@@ -51,7 +51,7 @@ int zone_load_contents(conf_t *conf, const knot_dname_t *zone_name,
 	zl.err_handler = &handler;
 	zl.creator->master = !zone_load_can_bootstrap(conf, zone_name);
 
-	*contents = zonefile_load(&zl);
+	*contents = zonefile_load(&zl, 0);
 	zonefile_close(&zl);
 	if (*contents == NULL) {
 		return KNOT_ERROR;
