@@ -42,6 +42,24 @@ typedef struct knot_creds {
 	uint8_t peer_pin[];
 } knot_creds_t;
 
+_public_
+const char *knot_tls_priority(bool tls12)
+{
+#if GNUTLS_VERSION_NUMBER >= 0x030702
+#define TLS_COMPAT      "%DISABLE_TLS13_COMPAT_MODE:"
+#else
+#define TLS_COMPAT      ""
+#endif
+
+#define COMMON_PRIORITY "-VERS-ALL:+VERS-TLS1.3:" \
+                        TLS_COMPAT \
+                        "-GROUP-ALL:+GROUP-X25519:+GROUP-SECP256R1:" \
+                                   "+GROUP-SECP384R1:+GROUP-SECP521R1"
+#define TLS12           ":+VERS-TLS1.2"
+
+	return tls12 ? COMMON_PRIORITY TLS12 : COMMON_PRIORITY;
+}
+
 static int tls_anti_replay_db_add_func(void *dbf, time_t exp_time,
                                        const gnutls_datum_t *key,
                                        const gnutls_datum_t *data)
@@ -362,19 +380,22 @@ _public_
 int knot_tls_session(struct gnutls_session_int **session,
                      struct knot_creds *creds,
                      struct gnutls_priority_st *priority,
-                     const char *alpn,
+                     bool quic,
                      bool early_data,
                      bool server)
 {
-	if (session == NULL || creds == NULL || priority == NULL || alpn == NULL) {
+	if (session == NULL || creds == NULL || priority == NULL) {
 		return KNOT_EINVAL;
 	}
 
+	const char *alpn = quic ? "\x03""doq" : "\x03""dot";
 	gnutls_init_flags_t flags = GNUTLS_NO_SIGNAL;
 	if (early_data) {
 		flags |= GNUTLS_ENABLE_EARLY_DATA;
 #ifdef ENABLE_QUIC // Next flags aren't available in older GnuTLS versions.
-		flags |= GNUTLS_NO_AUTO_SEND_TICKET | GNUTLS_NO_END_OF_EARLY_DATA;
+		if (quic) {
+			flags |= GNUTLS_NO_END_OF_EARLY_DATA;
+		}
 #endif
 	}
 
